/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.team.TeamMemberReq;
import com.zayac.admin.resp.MemberPagination;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamMember;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.model.resp.UserWithRolesAndAccountsResp;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;

/**
 * 新注册处理相关逻辑
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class RegistrationService {
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final TelegramMessageService telegramMessageService;

    public CompletableFuture<Void> processRegistration(UserWithRolesAndAccountsResp minister,
                                                       AccountResp account,
                                                       Map<String, UserWithRolesAndAccountsResp> accountUsernameToUserMap,
                                                       Map<String, TeamAccount> teamAccountMap,
                                                       Team currentTeamInfo,
                                                       Team prevTeamInfo,
                                                       LocalDate nowDate,

                                                       Executor asyncTaskExecutor) {
        if (prevTeamInfo != null && currentTeamInfo.getSubMemberCount() > prevTeamInfo.getSubMemberCount()) {
            int registerCount = currentTeamInfo.getSubMemberCount() - prevTeamInfo.getSubMemberCount();
            TeamMemberReq memberListReq = TeamMemberReq.builder()
                .registerStartDate(nowDate)
                .registerEndDate(nowDate)
                .startDate(nowDate)
                .endDate(nowDate)
                .registerSort(1)
                .pageSize(registerCount)
                .build();
            CompletableFuture<MemberPagination<List<TeamMember>>> memberPaginationCompletableFuture = completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                });
            return memberPaginationCompletableFuture.thenApplyAsync(MemberPagination::getList, asyncTaskExecutor)
                .thenAcceptAsync(members -> {
                    log.info("Successfully get [{}] new registered members", members.size());
                    if (CollUtil.isNotEmpty(members)) {
                        Map<String, List<TeamMember>> groupByTopAgentName = members.stream()
                            .collect(Collectors.groupingBy(TeamMember::getTopAgentName));
                        groupByTopAgentName.forEach((accountName, accountMembers) -> {
                            String notification = telegramMessageService
                                .buildRegistrationMessage(accountMembers, teamAccountMap.get(accountName));
                            var currUser = accountUsernameToUserMap.get(accountName);
                            if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                                String botToken = StrUtil.isEmpty(currUser.getBotToken())
                                    ? minister.getBotToken()
                                    : currUser.getBotToken();
                                telegramMessageService.sendMessage(botToken, currUser.getRegAndDepIds(), notification);
                            }
                            telegramMessageService
                                .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, notification);
                        });
                    }
                }, asyncTaskExecutor);
        }
        return CompletableFuture.completedFuture(null);
    }
}
