/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.service;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.json.JSONUtil;
import com.zayac.admin.resp.ApiResponse;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;
import reactor.util.retry.Retry;
import top.continew.starter.core.exception.BusinessException;

import java.time.Duration;
import java.util.Map;
import java.util.concurrent.Semaphore;

/**
 * 封装了一个简单可用的调用后端api的服务类
 */
@Service
public class WebClientService {
    private final WebClient webClient;

    private final Semaphore semaphore;

    public WebClientService(@Value("${webclient.max-concurrent-requests}") int maxConcurrentRequests) {
        this.webClient = WebClient.create();
        this.semaphore = new Semaphore(maxConcurrentRequests);
    }

    public <T> T fetchData(String url,
                           String headers,
                           Object params,
                           ParameterizedTypeReference<ApiResponse<T>> typeRef) {
        try {
            semaphore.acquire();  // 尝试获取许可
            return this.webClient.post().uri(url).headers(httpHeaders -> {
                Map<String, String> headerMap = JSONUtil.toBean(headers, new TypeReference<>() {
                }, true);
                headerMap.forEach(httpHeaders::add);
            })
                .body(params != null ? BodyInserters.fromValue(params) : BodyInserters.empty())
                .retrieve()
                .onStatus(status -> status.is4xxClientError() || status.is5xxServerError(), response -> response
                    .bodyToMono(String.class)
                    .map(body -> new BusinessException("Error response: " + body)))
                .bodyToMono(typeRef)
                .flatMap(this::respHandler)
                .retryWhen(Retry.backoff(3, Duration.ofSeconds(3)))
                .block(); // 遇到网络问题,每三秒重试一次
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            throw new BusinessException("Failed to acquire semaphore", e);
        } finally {
            semaphore.release();  // 释放许可
        }
    }

    public <T> Mono<T> MonoFetchData(String url,
                                     String headers,
                                     Object params,
                                     ParameterizedTypeReference<ApiResponse<T>> typeRef) {
        System.out.println(Thread.currentThread().getName());
        System.out.println(Thread.currentThread().getId());
        return Mono.fromCallable(() -> {
            semaphore.acquire();  // 尝试获取许可
            return true;
        })
            .subscribeOn(Schedulers.boundedElastic())
            .flatMap(acquired -> this.webClient.post().uri(url).headers(httpHeaders -> {
                Map<String, String> headerMap = JSONUtil.toBean(headers, new TypeReference<>() {
                }, true);
                headerMap.forEach(httpHeaders::add);
            })
                .body(params != null ? BodyInserters.fromValue(params) : BodyInserters.empty())
                .retrieve()
                .onStatus(status -> status.is4xxClientError() || status.is5xxServerError(), response -> response
                    .bodyToMono(String.class)
                    .map(body -> new BusinessException("Error response: " + body)))
                .bodyToMono(typeRef)
                .flatMap(this::monoRespHandler)
                .retryWhen(Retry.backoff(3, Duration.ofSeconds(3)))
                .doFinally(signal -> semaphore.release()));
    }

    private <T> Mono<T> monoRespHandler(ApiResponse<T> apiResponse) {
        // 根据响应状态码处理逻辑
        if (apiResponse.getStatusCode() == 6000) {
            return Mono.just(apiResponse.getData());  // 正常情况下返回数据
        } else if (apiResponse.getStatusCode() == 6001) {
            return Mono.error(new BusinessException("API token expired"));  // API令牌过期
        } else {
            return Mono.error(new BusinessException("Error status code: " + apiResponse.getStatusCode()));  // 其他错误
        }
    }

    private <T> Mono<T> respHandler(ApiResponse<T> apiResponse) {
        System.out.println(apiResponse);
        //api接口只有在statusCode为6000的时候返回的数据才是正常数据
        if (apiResponse.getStatusCode().equals(6000)) {
            return Mono.just(apiResponse.getData());
        } else if (apiResponse.getStatusCode().equals(6001)) {
            // TODO 调用登录接口,刷新header信息重试
            return Mono.error(new BusinessException("xApiToken失效"));  // 状态码6001，登录失效
        } else {
            return Mono.error(new BusinessException("错误状态码: " + apiResponse.getStatusCode()));  // 其他状态码，抛出异常
        }
    }
}